#!/usr/local/bin/perl

use 5.010;
use strict;
use warnings;

use File::Compare;
use File::Fetch;
use File::Share qw(dist_dir dist_file);
use File::Spec::Functions;
use File::Temp qw(tempdir);
use Getopt::Long;
use IO::Uncompress::Bunzip2 qw(bunzip2 $Bunzip2Error) ;
use IO::Uncompress::Gunzip qw(gunzip $GunzipError) ;
use Net::Patricia::ipvXbycc; # for File::Share

my $verbose;
GetOptions("verbose" => \$verbose);

{
    my $share_dir = dist_dir 'Net-Patricia-ipvXbycc';
    my $tmp_dir   = tempdir( TEMPLATE => catdir($share_dir, 'ipv4bycc.cidr.XXXX'), CLEANUP => 1 );
    my $zip       = catfile $tmp_dir, "cidr.txt.gz";
    my $pla       = catfile $tmp_dir, "cidr.txt";
    my $txt       = catfile $share_dir, 'ipv4bycc.cidr.txt';

    my $ff = File::Fetch->new(uri => "http://nami.jp/ipv4bycc/cidr.txt.gz");
    $ff->fetch( to => $tmp_dir ) or die $ff->error;

    gunzip $zip => $pla or die "gunzip failed: $GunzipError";

    if ( compare $pla, $txt ) {
        rename $pla, $txt or die "rename failed.";
        say "$txt is updated." if $verbose;
        my $mtime = (stat($zip))[9];
        utime $mtime, $mtime, $txt;
    }
    else {
        say "$txt is already up to date." if $verbose;
    }
}

{
    my $share_dir = dist_dir 'Net-Patricia-ipvXbycc';
    my $tmp_dir   = tempdir( TEMPLATE => catdir($share_dir, 'ipv6bycc.cidr.XXXX'), CLEANUP => 1 );
    my $zip       = catfile $tmp_dir, "ipv6_cidr.txt.bz2";
    my $pla       = catfile $tmp_dir, "ipv6_cidr.txt";
    my $txt       = catfile $share_dir, 'ipv6bycc.cidr.txt';

    my $ff = File::Fetch->new(uri => "https://foxmaster.jp/ipv6bycc/ipv6_cidr.txt.bz2");
    $ff->fetch( to => $tmp_dir ) or die $ff->error;

    bunzip2 $zip => $pla or die "bunzip2 failed: $Bunzip2Error";

    if ( compare $pla, $txt ) {
        rename $pla, $txt or die "rename failed.";
        say "$txt is updated." if $verbose;
        my $mtime = (stat($zip))[9];
        utime $mtime, $mtime, $txt;
    }
    else {
        say "$txt is already up to date." if $verbose;
    }
}
